@@ -128,7 +128,7 @@ public class SAMTextHeaderCodec {
             return null;
         }
         mCurrentLine = mReader.readLine();
-        textHeader.append(mCurrentLine).append("\n");
+        textHeader.append(mCurrentLine).append('\n');
         return mCurrentLine;
     }
 
