@@ -81,7 +81,7 @@ public class MetricsFile<BEAN extends MetricBase, HKEY extends Comparable> imple
 
     /** Returns the histogram contained in the metrics file if any. */
     public Histogram<HKEY> getHistogram() {
-        if (histograms.size() > 0) return this.histograms.get(0);
+        if (!histograms.isEmpty()) return this.histograms.get(0);
         else return null;
     }
 
@@ -382,7 +382,7 @@ public class MetricsFile<BEAN extends MetricBase, HKEY extends Comparable> imple
 
                             for (int i=0; i<fields.length; ++i) {
                                 Object value = null;
-                                if (values[i] != null && values[i].length() > 0) {
+                                if (values[i] != null && !values[i].isEmpty()) {
                                     value = formatter.parseObject(values[i], fields[i].getType());
                                 }
 
@@ -471,13 +471,13 @@ public class MetricsFile<BEAN extends MetricBase, HKEY extends Comparable> imple
             if (tryOtherPackages) {
                 for (final String p : packages) {
                     try {
-                        return loadClass(p + className.substring(className.lastIndexOf(".")), false);
+                        return loadClass(p + className.substring(className.lastIndexOf('.')), false);
                     }
                     catch (ClassNotFoundException cnf2) {/* do nothing */}
                     // If it ws an inner class, try and see if it's a stand-alone class now
-                    if (className.indexOf("$") > -1) {
+                    if (className.indexOf('$') > -1) {
                         try {
-                            return loadClass(p + "." + className.substring(className.lastIndexOf("$") + 1), false);
+                            return loadClass(p + "." + className.substring(className.lastIndexOf('$') + 1), false);
                         }
                         catch (ClassNotFoundException cnf2) {/* do nothing */}
                     }
