@@ -46,9 +46,46 @@ import java.util.regex.Pattern;
 public class SequenceUtil {
     /** Byte typed variables for all normal bases. */
     public static final byte a = 'a', c = 'c', g = 'g', t = 't', n = 'n', A = 'A', C = 'C', G = 'G', T = 'T', N = 'N';
+
     public static final byte[] VALID_BASES_UPPER = new byte[]{A, C, G, T};
     public static final byte[] VALID_BASES_LOWER = new byte[]{a, c, g, t};
 
+    private static final byte A_MASK = 1;
+    private static final byte C_MASK = 2;
+    private static final byte G_MASK = 4;
+    private static final byte T_MASK = 8;
+
+    private static final byte[] bases = new byte[127];
+
+    /*
+     * Definition of IUPAC codes:
+     * http://www.bioinformatics.org/sms2/iupac.html
+     */
+    static {
+        Arrays.fill(bases, (byte) 0);
+        bases[A] = A_MASK;
+        bases[C] = C_MASK;
+        bases[G] = G_MASK;
+        bases[T] = T_MASK;
+        bases['M'] = A_MASK | C_MASK;
+        bases['R'] = A_MASK | G_MASK;
+        bases['W'] = A_MASK | T_MASK;
+        bases['S'] = C_MASK | G_MASK;
+        bases['Y'] = C_MASK | T_MASK;
+        bases['K'] = G_MASK | T_MASK;
+        bases['V'] = A_MASK | C_MASK | G_MASK;
+        bases['H'] = A_MASK | C_MASK | T_MASK;
+        bases['D'] = A_MASK | G_MASK | T_MASK;
+        bases['B'] = C_MASK | G_MASK | T_MASK;
+        bases['N'] = A_MASK | C_MASK | G_MASK | T_MASK;
+        // Also store the bases in lower case
+        for (int i = 'A'; i <= 'Z'; i++) {
+            bases[(byte) i + 32] = bases[(byte) i];
+        }
+        bases['.'] = A_MASK | C_MASK | G_MASK | T_MASK;
+    };
+
+
     /**
      * Calculate the reverse complement of the specified sequence
      * (Stolen from Reseq)
@@ -62,15 +99,27 @@ public class SequenceUtil {
         return htsjdk.samtools.util.StringUtil.bytesToString(bases);
     }
 
-    /** Attempts to efficiently compare two bases stored as bytes for equality. */
-    public static boolean basesEqual(byte lhs, byte rhs) {
-        if (lhs == rhs) return true;
-        else {
-            if (lhs > 90) lhs -= 32;
-            if (rhs > 90) rhs -= 32;
-        }
 
-        return lhs == rhs;
+    /**
+     * Efficiently compare two IUPAC base codes, simply returning true if they are equal (ignoring case),
+     * without considering the set relationships between ambiguous codes.
+     */
+    public static boolean basesEqual(final byte lhs, final byte rhs) {
+        return (bases[lhs] == bases[rhs]);
+    }
+
+    /**
+     * Efficiently compare two IUPAC base codes, one coming from a read sequence and the other coming from
+     * a reference sequence, using the reference code as a 'pattern' that the read base must match.
+     *
+     * We take ambiguous codes into account, returning true if the set of possible bases
+     * represented by the read value is a (non-strict) subset of the possible bases represented
+     * by the reference value.
+     *
+     * Since the comparison is directional, make sure to pass read / ref codes in correct order.
+     */
+    public static boolean readBaseMatchesRefBaseWithAmbiguity(final byte readBase, final byte refBase) {
+        return (bases[readBase] & bases[refBase]) == bases[readBase];
     }
 
     /**
@@ -82,10 +131,11 @@ public class SequenceUtil {
 
     /** Returns true if the byte is in [acgtACGT]. */
     public static boolean isValidBase(final byte b) {
-        for (final byte validBase : VALID_BASES_UPPER) {
-            if (b == validBase) return true;
-        }
-        for (final byte validBase : VALID_BASES_LOWER) {
+        return isValidBase(b, VALID_BASES_UPPER) || isValidBase(b, VALID_BASES_LOWER);
+    }
+
+    private static boolean isValidBase(final byte b, final byte[] validBases) {
+        for (final byte validBase : validBases) {
             if (b == validBase) return true;
         }
         return false;
@@ -307,6 +357,27 @@ public class SequenceUtil {
         return Integer.toString(clipLength) + "S";
     }
 
+    /**
+     * Helper method to handle the various use cases of base comparison.
+     *
+     * @param readBase the read base to match
+     * @param refBase the reference base to match
+     * @param negativeStrand set to true if the base to test is on the negative strand and should be reverse complemented (only applies if bisulfiteSequence is true)
+     * @param bisulfiteSequence set to true if the base to match is a bisulfite sequence and needs to be converted
+     * @param matchAmbiguousRef causes the match to return true when the read base is a subset of the possible IUPAC reference bases, but not the other way around
+     * @return true if the bases match, false otherwise
+     */
+    private static boolean basesMatch(final byte readBase, final byte refBase, final boolean negativeStrand,
+                                      final boolean bisulfiteSequence, final boolean matchAmbiguousRef) {
+        if (bisulfiteSequence) {
+            if (matchAmbiguousRef) return bisulfiteBasesMatchWithAmbiguity(negativeStrand, readBase, refBase);
+            else return bisulfiteBasesEqual(negativeStrand, readBase, refBase);
+        } else {
+            if (matchAmbiguousRef) return readBaseMatchesRefBaseWithAmbiguity(readBase, refBase);
+            else return basesEqual(readBase, refBase);
+        }
+    }
+
     /** Calculates the number of mismatches between the read and the reference sequence provided. */
     public static int countMismatches(final SAMRecord read, final byte[] referenceBases) {
         return countMismatches(read, referenceBases, 0, false);
@@ -328,8 +399,12 @@ public class SequenceUtil {
      *                          and C->T on the positive strand and G->A on the negative strand will not be counted
      *                          as mismatches.
      */
+    public static int countMismatches(final SAMRecord read, final byte[] referenceBases, final int referenceOffset, final boolean bisulfiteSequence) {
+        return countMismatches(read, referenceBases, referenceOffset, bisulfiteSequence, false);
+    }
+
     public static int countMismatches(final SAMRecord read, final byte[] referenceBases, final int referenceOffset,
-                                      final boolean bisulfiteSequence) {
+                                      final boolean bisulfiteSequence, final boolean matchAmbiguousRef) {
         try {
             int mismatches = 0;
 
@@ -341,15 +416,9 @@ public class SequenceUtil {
                 final int length = block.getLength();
 
                 for (int i = 0; i < length; ++i) {
-                    if (!bisulfiteSequence) {
-                        if (!basesEqual(readBases[readBlockStart + i], referenceBases[referenceBlockStart + i])) {
-                            ++mismatches;
-                        }
-                    } else {
-                        if (!bisulfiteBasesEqual(read.getReadNegativeStrandFlag(), readBases[readBlockStart + i],
-                                referenceBases[referenceBlockStart + i])) {
-                            ++mismatches;
-                        }
+                    if (!basesMatch(readBases[readBlockStart + i], referenceBases[referenceBlockStart + i],
+                            read.getReadNegativeStrandFlag(), bisulfiteSequence, matchAmbiguousRef)) {
+                        ++mismatches;
                     }
                 }
             }
@@ -528,16 +597,18 @@ public class SequenceUtil {
     }
 
     /**
-     * Calculates the for the predefined NM tag from the SAM spec. To the result of
-     * countMismatches() it adds 1 for each indel.
+     * Calculates the predefined NM tag from the SAM spec: (# of mismatches + # of indels)
+     * For the purposes for calculating mismatches, we do not yet support IUPAC ambiguous codes
+     * (see <code>readBaseMatchesRefBaseWithAmbiguity</code> method).
      */
     public static int calculateSamNmTag(final SAMRecord read, final byte[] referenceBases) {
         return calculateSamNmTag(read, referenceBases, 0, false);
     }
 
     /**
-     * Calculates the for the predefined NM tag from the SAM spec. To the result of
-     * countMismatches() it adds 1 for each indel.
+     * Calculates the predefined NM tag from the SAM spec: (# of mismatches + # of indels)
+     * For the purposes for calculating mismatches, we do not yet support IUPAC ambiguous codes
+     * (see <code>readBaseMatchesRefBaseWithAmbiguity</code> method).
      *
      * @param referenceOffset 0-based offset of the first element of referenceBases relative to the start
      *                        of that reference sequence.
@@ -548,8 +619,9 @@ public class SequenceUtil {
     }
 
     /**
-     * Calculates the for the predefined NM tag from the SAM spec. To the result of
-     * countMismatches() it adds 1 for each indel.
+     * Calculates the predefined NM tag from the SAM spec: (# of mismatches + # of indels)
+     * For the purposes for calculating mismatches, we do not yet support IUPAC ambiguous codes
+     * (see <code>readBaseMatchesRefBaseWithAmbiguity</code> method).
      *
      * @param referenceOffset   0-based offset of the first element of referenceBases relative to the start
      *                          of that reference sequence.
@@ -559,7 +631,7 @@ public class SequenceUtil {
      */
     public static int calculateSamNmTag(final SAMRecord read, final byte[] referenceBases,
                                         final int referenceOffset, final boolean bisulfiteSequence) {
-        int samNm = countMismatches(read, referenceBases, referenceOffset, bisulfiteSequence);
+        int samNm = countMismatches(read, referenceBases, referenceOffset, bisulfiteSequence, false);
         for (final CigarElement el : read.getCigar().getCigarElements()) {
             if (el.getOperator() == CigarOperator.INSERTION || el.getOperator() == CigarOperator.DELETION) {
                 samNm += el.getLength();
@@ -569,7 +641,7 @@ public class SequenceUtil {
     }
 
     /**
-     * Attempts to calculate the for the predefined NM tag from the SAM spec using the cigar string alone.
+     * Attempts to calculate the predefined NM tag from the SAM spec using the cigar string alone.
      * It may calculate incorrectly if ambiguous operators (Like M) are used.
      *
      * Needed for testing infrastructure: SAMRecordSetBuilder
@@ -658,9 +730,9 @@ public class SequenceUtil {
     }
 
     /**
-     * Returns true if the bases are equal OR if the mismatch cannot be accounted for by
-     * bisfulite treatment.  C->T on the positive strand and G->A on the negative strand
-     * do not count as mismatches
+     * Returns true if the bases are equal OR if the mismatch can be accounted for by
+     * bisulfite treatment. C->T on the positive strand and G->A on the negative strand
+     * do not count as mismatches.
      */
     public static boolean bisulfiteBasesEqual(final boolean negativeStrand, final byte read, final byte reference) {
         return (basesEqual(read, reference)) || (isBisulfiteConverted(read, reference, negativeStrand));
@@ -670,6 +742,15 @@ public class SequenceUtil {
         return bisulfiteBasesEqual(false, read, reference);
     }
 
+    /**
+     * Same as above, but use <code>readBaseMatchesRefBaseWithAmbiguity</code> instead of <code>basesEqual</code>.
+     * Note that <code>isBisulfiteConverted</code> is not affected because it only applies when the
+     * reference base is non-ambiguous.
+     */
+    public static boolean bisulfiteBasesMatchWithAmbiguity(final boolean negativeStrand, final byte read, final byte reference) {
+        return (readBaseMatchesRefBaseWithAmbiguity(read, reference)) || (isBisulfiteConverted(read, reference, negativeStrand));
+    }
+
     /**
      * Checks for bisulfite conversion, C->T on the positive strand and G->A on the negative strand.
      */
@@ -767,7 +848,7 @@ public class SequenceUtil {
                     boolean matched = match.find();
                     if (matched) {
                         String mg;
-                        if (((mg = match.group(1)) != null) && (mg.length() > 0)) {
+                        if (((mg = match.group(1)) != null) && (!mg.isEmpty())) {
                             // It's a number , meaning a series of matches
                             final int num = Integer.parseInt(mg);
                             for (int i = 0; i < num; i++) {
@@ -778,7 +859,7 @@ public class SequenceUtil {
                                 }
                                 basesMatched++;
                             }
-                        } else if (((mg = match.group(2)) != null) && (mg.length() > 0)) {
+                        } else if (((mg = match.group(2)) != null) && (!mg.isEmpty())) {
                             // It's a single nucleotide, meaning a mismatch
                             if (basesMatched < cigElLen) {
                                 ret[outIndex++] = StringUtil.charToByte(mg.charAt(0));
@@ -787,7 +868,7 @@ public class SequenceUtil {
                                 throw new IllegalStateException("Should never happen.");
                             }
                             basesMatched++;
-                        } else if (((mg = match.group(3)) != null) && (mg.length() > 0)) {
+                        } else if (((mg = match.group(3)) != null) && (!mg.isEmpty())) {
                             // It's a deletion, starting with a caret
                             // don't include caret
                             if (includeReferenceBasesForDeletions) {
@@ -889,7 +970,7 @@ public class SequenceUtil {
     }
 
     /**
-     * A rip off samtools bam_md.c
+     * Calculate MD and NM similarly to Samtools, except that N->N is a match.
      *
      * @param record
      * @param ref
@@ -980,7 +1061,7 @@ public class SequenceUtil {
     public static List<byte[]> generateAllKmers(final int length) {
         final List<byte[]> sofar = new LinkedList<byte[]>();
 
-        if (sofar.size() == 0) {
+        if (sofar.isEmpty()) {
             sofar.add(new byte[length]);
         }
 
