@@ -68,7 +68,7 @@ public abstract class Genotype implements Comparable<Genotype>, Serializable {
 
     protected Genotype(final String sampleName, final String filters) {
         this.sampleName = sampleName;
-        this.filters = filters;
+        this.filters = filters == null || filters.isEmpty() ? null : filters;
     }
 
     /**
@@ -637,9 +637,9 @@ public abstract class Genotype implements Comparable<Genotype>, Serializable {
             return "";
         else {
             StringBuilder b = new StringBuilder();
-            b.append(" ").append(name).append(" ");
+            b.append(' ').append(name).append(' ');
             for ( int i = 0; i < vs.length; i++ ) {
-                if ( i != 0 ) b.append(",");
+                if ( i != 0 ) b.append(',');
                 b.append(vs[i]);
             }
             return b.toString();
@@ -663,4 +663,4 @@ public abstract class Genotype implements Comparable<Genotype>, Serializable {
     protected final static boolean isForbiddenKey(final String key) {
         return PRIMARY_KEYS.contains(key);
     }
-}
\ No newline at end of file
+}
