@@ -6,9 +6,9 @@ import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
-public class SRAReferenceTest {
+public class SRAReferenceTest extends AbstractSRATest {
     @DataProvider(name = "testReference")
-    public Object[][] createDataForReference() {
+    private Object[][] createDataForReference() {
         return new Object[][] {
                 {"SRR2096940", "CM000681.1", 95001, 95050, "AGATGATTCAGTCTCACCAAGAACACTGAAAGTCACATGGCTACCAGCAT"},
         };
@@ -16,10 +16,8 @@ public class SRAReferenceTest {
 
     @Test(dataProvider = "testReference")
     public void testReference(String acc, String refContig, int refStart, int refStop, String refBases) {
-        if (!SRAAccession.isSupported()) return;
-
-        ReferenceSequenceFile refSeqFile = new SRAIndexedSequenceFile(new SRAAccession(acc));
-        ReferenceSequence refSeq = refSeqFile.getSubsequenceAt(refContig, refStart, refStop);
+        final ReferenceSequenceFile refSeqFile = new SRAIndexedSequenceFile(new SRAAccession(acc));
+        final ReferenceSequence refSeq = refSeqFile.getSubsequenceAt(refContig, refStart, refStop);
         Assert.assertEquals(new String(refSeq.getBases()), refBases);
     }
 }
